// $Id: Container.cc,v 1.22 2001/04/28 17:51:31 rob Exp $

#include "Container.h"

// This is the baseclass of everything that has a window.  It isn't
// really designed to be instantiated by itself, although it can be.
// In isn't useful at all by itself, though.

// Stuff should subclass this, and at a minimum should override the
// following methods: paint() and setPreferredSize().  update() isn't
// really used yet; it will be for drawing without erasing the window.

bool Container::initializedX=false;
string Container::displayName="";
Display *Container::display=NULL;
Window Container::root=0;
int Container::screen=0;
Colormap Container::colormap=0;
//XColor Container::border, Container::background, Container::foreground;
Atom Container::wm_del_win=0;
WindowMap Container::windowToContainer;
string Container::borderColorName = "black";
string Container::backgroundColorName = "white";
string Container::foregroundColorName = "black";
string Container::defaultFontName = DEFAULT_FONT;

Container::Container() {
  this->x=0;
  this->y=0;
  this->w=10;
  this->h=10;
  setParent(NULL);	
  _init();
}

Container::Container(int w, int h) {
  this->x=0;
  this->y=0;
  this->w=w;
  this->h=h;
  setParent(NULL);	
  _init();
}

Container::Container(Container *parent) {
  parent->setLayout(this);
  setParent(parent);
  _init();
}

Container::Container(Container *parent, int w, int h) {
  this->x=0;
  this->y=0;
  this->w=w;
  this->h=h;
  setParent(parent);
  _init();
}

Container::Container(int x, int y, int w, int h) {
  this->x=x;
  this->y=y;
  this->w=w;
  this->h=h;
  setParent(NULL);	
  _init();
}

Container::Container(Container *parent, int x, int y, int w, int h) {
  this->x=x;
  this->y=y;
  this->w=w;
  this->h=h;
  setParent(parent);
  _init();
}

Container::~Container() {
  dprintf("in Container::~Container\n");
}

// Static methods

bool Container::isScreenOn() {
  bool power_on=true;
  
#ifdef USE_DPMS
  int major_version,minor_version;
  
    // The server must know about DPMS, and the monitor must be DPMS capable
    // in order for us to find out any other info about the power state of
    // the monitor
  if (DPMSQueryExtension(display, &major_version, &minor_version) &&
      DPMSCapable(display)) {
    
    CARD16 power_level;
    BOOL state;
    
    if (DPMSInfo(display,&power_level,&state)) {
#ifdef DEBUG
      char *label[4]= {"DPMSModeOn",
                       "DPMSModeStandby",
                       "DPMSModeSuspend",
                       "DPMSModeOff"};
      cerr << "dpms: power_level=" << label[power_level] << " state=" << (state?"enabled":"disabled") << endl;
#endif
      switch (power_level) {
        case DPMSModeStandby:
        case DPMSModeSuspend:
        case DPMSModeOff:
            // any one of these modes mean that some form of energy savings is
            // on, and the monitor is powered down to some extent.
          power_on=false;
        default:
          break;
      }
    }
  }
#endif
  return power_on;
}

void Container::getColor(XColor * xc, const char *xc_string) {
  if (XParseColor(display, colormap, xc_string, xc) == 0)
    fprintf(stderr, "Container: invalid color %s", xc_string);
  else
    XAllocColor(display, colormap, xc);
}

void Container::getColors() {
    /* Do the foreground and background colors. */
  getColor(&border, borderColorName.c_str());
  getColor(&background, backgroundColorName.c_str());
  getColor(&foreground, foregroundColorName.c_str());
}

GC Container::getGC(const char *colorName) {
  XGCValues gcv;
  XColor color;
	
  getColor(&color,colorName);
	
  gcv.font = font->fid;
  gcv.foreground = color.pixel;
  gcv.background = background.pixel;
  gcv.function = GXcopy;
  GC gc = XCreateGC(display, win,
    GCForeground | GCBackground | GCFunction | GCFont,
    &gcv);

  return gc;
}

// Apparently no equivalent to a static block in C++?  I have to call this
// at each constructor.
void Container::staticInit() {
  if (!initializedX) {
    initializedX=true;
			
    dprintf("Container::staticInit()\n");
			
    if (displayName.length()==0) {
      displayName = getenv("DISPLAY");
    }
			
    if (!(display = XOpenDisplay(displayName.c_str()))) {
      fprintf(stderr,"Can't open display");
      exit(0);
    }
			
      /* get display info */
    screen = DefaultScreen(display);
			
    root = DefaultRootWindow(display);
    XSelectInput(display, root, PropertyChangeMask);
			
    wm_del_win = XInternAtom(display, "WM_DELETE_WINDOW", False);

    colormap = DefaultColormap(display, screen);
  }
}

Container *Container::getContainerByEvent(XEvent *e) {
  if (windowToContainer.count(e->xany.window)>0) {
    return windowToContainer[e->xany.window];
  }
  return NULL;
}

// Instance methods

void Container::_init() {
  staticInit();
	
  getColors();

  font_name=defaultFontName;
  font = XLoadQueryFont(display, font_name.c_str());
  if (!font) {
    font = XLoadQueryFont(display, DEFAULT_FONT);
    fprintf(stderr,"Failed loading font '%s'.  Trying default font.\n",font_name.c_str());
  }
  if (!font) {
    fprintf(stderr,"Failed loading default font.\n");
    exit(1);
  }
  font_height = font->max_bounds.ascent + font->max_bounds.descent;
  font_ascent = font->max_bounds.ascent;
  font_width = font->max_bounds.width;

#ifdef USE_TOOLTIP
  tooltip=NULL;
  tooltipRelative=NULL;
#endif // USE_TOOLTIP
  createWindow();
}

void Container::createWindow() {
  Window parentWin;
  if (parent) {
    parentWin=parent->win;
  }
  else {
    parentWin=root;
  }
	
  win = XCreateSimpleWindow(display, parentWin, x, y, w, h,
    BORDER_WIDTH, border.pixel, background.pixel);
  windowToContainer[win]=this;

  mapWhenRealized=true;
	
  XSelectInput(display, win, MW_EVENTS);

  char temp[64];
  sprintf(temp,"%lx",(long)this);
  name=temp;
}


void Container::setParent(Container *parent)
{
  this->parent=parent;

  if (parent==NULL) {
    this->firstChild=NULL;
    this->lastChild=NULL;
    this->prevSibling=NULL;
    this->nextSibling=NULL;
  }
  else { // add this winder to the list managed by the parent
    this->firstChild=NULL;
    this->lastChild=NULL;
    if (!parent->firstChild) {
      parent->firstChild=this;
      this->prevSibling=NULL;
    }
    else {
      this->prevSibling=parent->lastChild;
      this->prevSibling->nextSibling=this;
    }
    this->nextSibling=NULL;
    parent->lastChild=this;
  }
}

Container *Container::getFirstChild() 
{
  return firstChild;
}

Container *Container::getNextSibling() 
{
  return nextSibling;
}


// This is a dummy virtual function designed to be overridden
void Container::setLayout(Container *target) 
{
  target->x=0;
  target->y=0;
  target->w=10;
  target->h=10;
}

// Another virtual function designed to be called by the Container
// class.  Width or height can be adjusted depending on the dir flag.  If
// dir is VERTICAL, the container is expecting the height to be adjusted.
void Container::setPreferredSize(int dir) 
{
  dprintf("in Container::setPreferredSize\n");

  if (dir==VERTICAL) {
    h=w*2; // default to rectangular so we can debug it.
  }
  else {
    w=h*2;
  }
}

void Container::resize() 
{
  XMoveResizeWindow(display, win, x, y, w, h);
}

void Container::realize()
{
  dprintf("in Container::realize\n");
  XGCValues gcv;

  gcv.font = font->fid;
  gcv.foreground = foreground.pixel;
  gcv.background = background.pixel;
  gcv.function = GXcopy;
  drawGC = XCreateGC(display, win,
    GCForeground | GCBackground | GCFunction | GCFont,
    &gcv);
	
  gcv.foreground = background.pixel;
  gcv.background = foreground.pixel;
  eraseGC = XCreateGC(display, win,
    GCForeground | GCBackground | GCFont, &gcv);
	
  if (mapWhenRealized) {
    map();
  }
}

void Container::startUnmapped() {
  mapWhenRealized=false;
}

void Container::map() {
  XMapWindow(display,win);
}

// map a window using coordinates relative to another window
void Container::map(Window relative, int width) {
  map();
}

void Container::unmap() {
  XUnmapWindow(display,win);
}

void Container::erase()
{
  XClearWindow(display, win);
}


// default paint draws a big ol' X to show that it is actually working.
// It is meant to be overridden...
void Container::paint()
{
  dprintf("in Container::paint\n");

  XSegment lines[100];

  lines[0].x1=0;
  lines[0].y1=0;
  lines[0].x2=w-1;
  lines[0].y2=h-1;
  lines[1].x1=0;
  lines[1].y1=h-1;
  lines[1].x2=w-1;
  lines[1].y2=0;
	
  XDrawSegments(display, win, drawGC, lines, 2);
}

#ifdef USE_TOOLTIP
void Container::setTooltip(Container *t) {
  tooltip=t;
}

void Container::setTooltip(Container *t, Container *relative) {
  tooltip=t;
  tooltipRelative=relative;
}
#endif // USE_TOOLTIP

void Container::handleButtonPress(XButtonEvent *e) {
  dprintf("in Container::handleButtonPress\n");
}

// Note that mouse motion events aren't enabled by default in the
// event mask.  One of the MotionMask event masks should be selected
// by XSelectEvent before this method (or an overridden subclass's
// method) will get called.
void Container::handleMouseMotion(XMotionEvent *e) {
  dprintf("in Container::handleMouseMotion\n");
}

void Container::handleKeyPress(XKeyEvent *e) {
  dprintf("in Container::handleKeyPress\n");
}

void Container::handleSelectionRequest(XSelectionRequestEvent *e) {
  dprintf("in Container::handleSelectionRequest\n");
}

void Container::handleSelectionClear(XSelectionClearEvent *e) {
  dprintf("in Container::handleSelectionClear\n");
}

void Container::handleEnterNotify(XCrossingEvent *e) {
  dprintf("in Container::handleEnterNotify\n");

#ifdef USE_TOOLTIP
  if (tooltip) {
    if (tooltipRelative) {
      tooltip->map(tooltipRelative->getWindow(),w);
    }
    else {
      tooltip->map(win,w);
    }
  }
#endif // USE_TOOLTIP
}

void Container::handleLeaveNotify(XCrossingEvent *e) {
  dprintf("in Container::handleLeaveNotify\n");

#ifdef USE_TOOLTIP
  if (tooltip) {
    tooltip->unmap();
  }
#endif // USE_TOOLTIP
}

void Container::drawCenteredText(GC gc,int x_center, int baseline, char *str, int count) {
  XCharStruct xc;
  int dum;
  XTextExtents(font,str,count,&dum,&dum,&dum,&xc);
	
  int x = x_center - xc.width/2;
  XDrawString(display, win, drawGC, x, baseline, str, count);
}

char *Container::toString() {
  return (char *)name.c_str();
}

