// $Id: CutBuffer.cc,v 1.8 2001/03/03 22:02:34 rob Exp $

#include "CutBuffer.h"

// There are 8 cut buffers supplied by the X Server; typically most
// users don't even know about them.  I use them all the time to store
// commands, email addresses, web addresses, and more.

// CUT_BUFFER0 is the default buffer where most applications place
// their selections that are marked by highlighting regions.

int CutBuffer::buffer_count=0;
CutBuffer *CutBuffer::selected=NULL;
CutBuffer *CutBuffer::first=NULL;

// Note: this isn't decended from TimedContainer because it is purely event
// driven.  There is no need to update this after some time interval; an X
// event will always be generated when something in a cut buffer changes.
CutBuffer::CutBuffer(Container *parent, int rows): TextBox(parent,rows) {
  dprintf("in CutBuffer::CutBuffer\n");

  allocAtom();
  if (!first) {
    first=this;
  }
  else {
    CutBuffer *c=first;
    while (c->next) {
      c=c->next;
    }
    c->next=this;
  }
  next=NULL;
}

CutBuffer *CutBuffer::getCurrentCutBuffer() {
  return selected;
}

// must tie an Atom to the cut buffer
void CutBuffer::allocAtom() {
  sprintf(atom_name,"CUT_BUFFER%d",buffer_count);
  buffer_count++;
  atom = XInternAtom(display, atom_name, (Bool) False);

  getBuffer();
}
	
void CutBuffer::setHighlight(bool highlight) {
  TextBox::setHighlight(highlight);
  if (highlight) {
    selected=this;
  }
  else {
    selected=NULL;
  }
}

// Get the data from the CUT_BUFFER? atom and set the text of this
// CutBuffer ojbect so that it will get displayed in the window.
void CutBuffer::getBuffer() {
  Atom actl_type;
  int actl_format;
  unsigned long nitems, after;
  char *data;

  if (text) XFree(text);
  setText(NULL,0);

  if (XGetWindowProperty(display, root, atom,
    0L, 10000000L, False, XA_STRING,
    &actl_type, &actl_format, &nitems,
    &after, (unsigned char **) &data) != Success)
    return;

  if (actl_type == XA_STRING && actl_format != 32) {
    setText(data,nitems);
		
    dprintf("CutBuffer::getBuffer: len=%ld data=%s\n", nitems, data);

  }
  else if (data)
    XFree(data);
}

// mark us as the current owner of the primary selection
void CutBuffer::setAsPrimaryBuffer(XButtonEvent *e) {
  if (!highlighted) {
    if (selected) {
      CutBuffer *old=selected;
			
      old->setHighlight(false);
      old->erase();
      old->paint();
    }
    XSetSelectionOwner(display, XA_PRIMARY, win, e->time);
    setHighlight(true);
    erase();
    paint();
  }
}

// Take the primary selection and stuff it into this cut buffer
void CutBuffer::pasteIntoBuffer() {
  if (XGetSelectionOwner(display, XA_PRIMARY) == None &&
    selected) {

    XChangeProperty(display, root, atom, XA_STRING,
      8, PropModeReplace,
      (unsigned char *) selected->text,
      (int) selected->length);
  }
  else {
    XConvertSelection(display, XA_PRIMARY, XA_STRING,
      atom, root, CurrentTime);
  }
}

// left button: take control of the primary selection away from
// another CutBuffer window, or from some other application
//
// middle button: paste the primary selection (from us or any other
// application) into the CutBuffer that we clicked on
void CutBuffer::handleButtonPress(XButtonEvent *e) {
  dprintf("in CutBuffer::handleButtonPress\n");

  if (e->button == 1) {
    setAsPrimaryBuffer(e);
  }
  else if (e->button == 2) {
    pasteIntoBuffer();
  }
}

// Check to see if we are the correct instance of the cut buffer to store
// the new data.  The X server has notified us that the data pointed to by
// this atom has changed, so we need to update our info.
void CutBuffer::handlePropertyNotify(XPropertyEvent *e) {
  dprintf("in CutBuffer::handlePropertyNotify\n");

  if (e->atom == atom) {
    getBuffer();
    erase();
    paint();
  }
}

// PropertyNotify events always occur on the main application window,
// not the window that is in the CutBuffer object itself.  So, we have
// to check each cut buffer for the correct atom
void CutBuffer::doPropertyNotify(XPropertyEvent *e) {
  CutBuffer *c=first;
  while (c) {
    c->handlePropertyNotify(e);
    c=c->next;
  }
}

// Someone wants the data contained in our cut buffer.  Better give it
// to them before they become very, very angry.
void CutBuffer::handleSelectionRequest(XSelectionRequestEvent *e) {
  dprintf("in CutBuffer::handleSelectionRequest\n");

  if (this==selected) {
    XSelectionEvent notify;

    notify.type = SelectionNotify;
    notify.display = e->display;
    notify.requestor = e->requestor;
    notify.selection = e->selection;
    notify.target = e->target;
    notify.property = None;
    notify.time = e->time;

    if (e->target == XA_STRING) {
      XChangeProperty(display, e->requestor, e->property,
        XA_STRING, 8, PropModeReplace,
        (unsigned char *) text, (int) length);
      notify.property = e->property;
    }
    else {
      dprintf("CutBuffer::handleSelectionRequest: target not recognized\n");
    }
		
    XSendEvent(display, e->requestor, False, 0, (XEvent *)&notify);
  }
  else {
    dprintf("CutBuffer::handleSelectionRequest: ERROR!  Container doesn't match requested\nwindow.\n");
  }
}

// Someone has taken the primary selection away from us.
void CutBuffer::handleSelectionClear(XSelectionClearEvent *e) {
  dprintf("in CutBuffer::handleSelectionClear\n");

  if (this==selected) {
    setHighlight(false);
    erase();
    paint();
  }
  else {
    dprintf("CutBuffer::handleSelectionClear: ERROR!  Container doesn't match requested\nwindow.\n");
  }
}

// Create the 8 cut buffers supplied by default by the X server.  The
// pointers to the CutBuffer objects are ignored, because they are never
// destroyed.
void CutBuffer::create(Container *parent, int rows) {
  int i;
  for (i=0; i<8; i++) {
      // ignore return value because we never destroy
    new CutBuffer(parent,rows);
  }
}
