// $Id: Mixer.cc,v 1.4 2001/03/03 22:02:34 rob Exp $

#include "Mixer.h"

// constructor for a soundcard volume controller.  Only one control is
// implemented per Mixer object, so several of these objects will
// probably be instantiated.

// Currently, the stereo capability of any mixers are ignored; the
// left and right channels are set to the same volume level.

const char *Mixer::labels[] = SOUND_DEVICE_LABELS;
const char *Mixer::names[] = SOUND_DEVICE_NAMES;

Mixer::Mixer(Container *parent, int tc, int device): BarGraph(parent,tc) {
  dprintf("in Mixer::Mixer\n");

  this->device=device;
  setLabel((char *)labels[device]);
}

void Mixer::realize() {
  XSelectInput(display, win, MW_EVENTS|Button1MotionMask);
  BarGraph::realize();
}

void Mixer::prepare() {
  dprintf("in Mixer::prepare\n");

  bar.cur=getVolume();
}

int Mixer::getVolume() {
  int fd=open("/dev/mixer",O_RDONLY);
  if (fd==-1) return 0;
	
  int vol;
  if (ioctl(fd, MIXER_READ(device), &vol) == -1) {
      // This should never happen: an undefined mixer channel was
      // accessed...
    vol=0;
  }
  close(fd);

  int right,left;
  right=vol&0xff;
  left=vol>>8;
  dprintf("in Mixer::getVolume left=%d right=%d\n",left,right);
  return (left+right)/2;
}

void Mixer::setVolume(int left, int right) {
  int fd=open("/dev/mixer",O_RDONLY);
  if (fd==-1) return;
	
  if (left<0) left=0;
  else if (left>100) left=100;
  if (right<0) right=0;
  else if (right>100) right=100;
  int vol=left<<8|right;
  if (ioctl(fd, MIXER_WRITE(device), &vol) == -1) {
      // This should never happen: an undefined mixer channel was
      // accessed...
    vol=0;
  }
  else {
    right=vol&0xff;
    left=vol>>8;
    bar.cur=(left+right)/2;
    paint();
  }
  close(fd);
}

void Mixer::handleButtonPress(XButtonEvent *e) {
  dprintf("in Mixer::handleButtonPress: x=%d\n",e->x);
  int left=e->x*100/w;
  setVolume(left,left);
}

// Because XSelectInput (above) uses Button1MotionMask, we know that
// any time this method is called, the mouse is moving and the left
// button is down.
void Mixer::handleMouseMotion(XMotionEvent *e) {
  dprintf("in Mixer::handleMouseMotion: x=%d\n",e->x);
  int left=e->x*100/w;
  setVolume(left,left);
}

// search through all available mixers, and create a mixer for each
void Mixer::create(Container *parent, int tc) {
  int fd=open("/dev/mixer",O_RDONLY);
  if (fd==-1) return;
	
  int mask;
  if (ioctl(fd, SOUND_MIXER_READ_DEVMASK, &mask) == -1) {
      /* Mixer capability is not available - handle this gracefully ... */
    close(fd);
    return;
  }
  close(fd);

  int i;
  for (i=0; i<SOUND_MIXER_NRDEVICES; i++) {
    int bit=1<<i;
    if (mask&bit) {
      dprintf("Found volume mixer %s\n",labels[i]);
			
        // ignore return value because we never destroy
      new Mixer(parent,tc,i);
    }
  }
}

// search through the list of mixers specified in a text string, and
// only create those mixers
void Mixer::create(Container *parent, int tc, char *list) {
  int fd=open("/dev/mixer",O_RDONLY);
  if (fd==-1) return;

  int mask;
  if (ioctl(fd, SOUND_MIXER_READ_DEVMASK, &mask) == -1) {
      /* Mixer capability is not available - handle this gracefully ... */
    close(fd);
    return;
  }
  close(fd);

  char *copy=strdup(list);
  char *delim=" \t,";

  char *tok=strtok(copy,delim);
  while (tok) {
    dprintf("Searching for mixer %s\n",tok);
    int i=0;
    int bit=1;
    while (i<SOUND_MIXER_NRDEVICES) {
      if (strcmp(tok,names[i])==0) {
        if (mask&bit) {
          dprintf(" mixer %s available\n",names[i]);
					
            // ignore return value because we never destroy
          new Mixer(parent,tc,i);
        }
        break;
      }
      i++;
      bit<<=1;
    }
    tok=strtok(NULL,delim);
  }
  free(copy);
}
