// $Id: xstatuspanel.cc,v 1.20 2001/05/14 02:34:19 rob Exp $

#include "config.h"
#include "debug.h"

#include <stdlib.h>
#include <stdio.h>
#include <fstream>

#include "Container.h"
#include "RowManager.h"
#include "TimedContainer.h"
#include "AnalogClock.h"
#include "BarGraph.h"
#include "DualBarGraph.h"
#include "Swap.h"
#include "NetPackets.h"
#include "TextBox.h"
#include "CutBuffer.h"
#include "Battery.h"
#include "Memory.h"
#include "Today.h"
#include "Disk.h"
#include "Chart.h"
#include "LoadAvg.h"
#include "CPU.h"
#include "Temperature.h"
#ifdef USE_TOOLTIP
#include "Tooltip.h"
#endif
#ifdef USE_CALENDAR
#include "Calendar.h"
#endif
#ifdef USE_MAIL
#include "Mail.h"
#include "UnixMail.h"
#include "ImapMail.h"
#endif
#ifdef USE_MIXER
#include "Mixer.h"
#endif

char *appName="xstatuspanel";
char *appClass="XStatusPanel";

char *progname="xstatuspanel";
char *geometry=NULL;
char *displayName=NULL;
int secondsPerClockTick=1;
int mailCheckTicks=300;
char *foregroundColorName=NULL;
char *backgroundColorName=NULL;
char *borderColorName=NULL;
char *fontName=DEFAULT_FONT;

char *defaults[]={
  "date",
  "clock",
  "load",
  "cpu",
  "memory",
  "swap",
  "net",
  "disk",
  "cut",
  NULL
};


void handlePropertyNotify(XPropertyEvent *e) {
  CutBuffer::doPropertyNotify(e);
}


void usage()
{
  typedef struct {
      char *option;
      char *description;
  } option_struct;
  option_struct options[]={
    {"-h","display this usage message"},
    {"-display <displayname>","X server to contact"},
    {"-geom <w>x<h>{+-}<x>{+-}<y>","specify window width & location"},
    {"-geometry","synonym for -geom"},
    {"-update <seconds>","number of seconds delay between updates"},
    {"-font <fontname>","default font"},
    {"-fg <colorname>","foreground color"},
    {"-bg <colorname>","background color"},
    {"-borderc <colorname>","color separating individual components"},
  };
	
  fprintf(stderr, "Usage: %s [-options ...]\n\nwhere options include:\n", progname);
  int num=sizeof(options)/sizeof(option_struct);
  for (int i=0; i<num; i++) {
    fprintf(stderr,"  %-35s%s\n",options[i].option,options[i].description);
  }
  exit(1);
}

void argProcess(int argc, char **argv) {
  progname=*argv++, argc--;

  while (argc > 0 && **argv == '-') {
    dprintf("argv[%d] = %s\n", argc, *argv);
    if (argc > 1) {
      if (strcmp(*argv, "-update") == 0) {
        ++argv, --argc;
        secondsPerClockTick = atoi(*argv);
        dprintf("update time = %d\n", secondsPerClockTick);
        if (secondsPerClockTick <= 0)
          usage();
      }
      else if (strcmp(*argv, "-display") == 0)
        displayName = *++argv, --argc;
      else if ((strcmp(*argv, "-geometry") == 0) ||
        (strcmp(*argv, "-geom") == 0))
        geometry = *++argv, --argc;
      else if (strcmp(*argv, "-fg") == 0)
        foregroundColorName = *++argv, --argc;
      else if (strcmp(*argv, "-bg") == 0)
        backgroundColorName = *++argv, --argc;
      else if (strcmp(*argv, "-borderc") == 0)
        borderColorName = *++argv, --argc;
      else if (strcmp(*argv, "-font") == 0)
        fontName = *++argv, --argc;
      else
        usage();
    }
    else {
      if (strcmp(*argv, "-h") == 0)
        usage();
      else
        usage();
    }
    ++argv, --argc;
  }
  if (displayName) {
    Container::setDisplay(displayName);
  }
  if (backgroundColorName) {
    Container::setDefaultBackground(backgroundColorName);
  }
  if (foregroundColorName) {
    Container::setDefaultForeground(foregroundColorName);
  }
  if (borderColorName) {
    Container::setDefaultBorderColor(borderColorName);
  }
  if (fontName) {
    Container::setDefaultFont(fontName);
  }
}

#ifdef USE_CALENDAR
void setCalendarTooltip(Container *c) {
    // statics to show if the Calendar tooltip has been added yet
  static Container *calendarParent=NULL;
  static Tooltip *calendar=NULL;

  if (!calendar) calendar=new Calendar();

  if (calendarParent) c->setTooltip(calendar,calendarParent);
  else {
    calendarParent=c;
    c->setTooltip(calendar);
  }
}
#endif // USE_CALENDAR

Container *process(Manager *w, char *line) {
  Container *c=NULL;
  char *delim=" \t";
  
  if ((!line) || (*line=='#')) return c;
  
  char *type=strtok(line,delim);
  if (!type) return c;
  
  if (strcmp(type,"-bg")==0) {
    dprintf("Found %s\n",type);
    char *color=strtok(NULL,delim);
    if (color) Container::setDefaultBackground(color);    
  }
  else if (strcmp(type,"-fg")==0) {
    dprintf("Found %s\n",type);
    char *color=strtok(NULL,delim);
    if (color) Container::setDefaultForeground(color);    
  }
  else if (strcmp(type,"-borderc")==0) {
    dprintf("Found %s\n",type);
    char *color=strtok(NULL,delim);
    if (color) Container::setDefaultBorderColor(color);    
  }
  else if (strcmp(type,"-font")==0) {
    dprintf("Found %s\n",type);
    char *name=strtok(NULL,delim);
    if (name) Container::setDefaultFont(name);    
  }
  else if (strcmp(type,"date")==0) {
    dprintf("Found %s\n",type);
    c=new Today(w,120);
#ifdef USE_CALENDAR
    setCalendarTooltip(c);
#endif
  }
  else if (strcmp(type,"clock")==0) {
    dprintf("Found %s\n",type);
    c=new AnalogClock(w,60);
#ifdef USE_CALENDAR
    setCalendarTooltip(c);
#endif
  }
  else if (strcmp(type,"battery")==0) {
    dprintf("Found %s\n",type);
    c=new Battery(w,1);
  }
  else if (strcmp(type,"load")==0) {
    dprintf("Found %s\n",type);
    c=new LoadAvg(w,5);
  }
  else if (strcmp(type,"cpu")==0) {
    dprintf("Found %s\n",type);
    CPU::create(w,1);
  }
  else if (strcmp(type,"memory")==0) {
    dprintf("Found %s\n",type);
    c=new Memory(w,1);
  }
  else if (strcmp(type,"swap")==0) {
    dprintf("Found %s\n",type);
    c=new Swap(w,1);
  }
  else if (strcmp(type,"net")==0) {
    dprintf("Found %s\n",type);
    NetPackets::create(w,1);
  }
  else if (strcmp(type,"disk")==0) {
    dprintf("Found %s\n",type);
    Disk::create(w,1);
  }
  else if (strncmp(type,"cut",3)==0) {
    dprintf("Found %s\n",type);
    CutBuffer::create(w,4);
    w->registerPropertyNotifyHandler(&handlePropertyNotify);
  }
#ifdef USE_MAIL
  else if (strcmp(type,"mail-spool")==0) {
    dprintf("Found %s\n",type);
    c=new UnixMail(w,mailCheckTicks);
    char *label=strtok(NULL,delim);
    if (label) ((Mail *)c)->setLabel(label);
  }
  else if (strcmp(type,"mail-imap")==0) {
    dprintf("Found %s\n",type);
    char *cfg=strtok(NULL,delim);
    char *label=strtok(NULL,delim);
    c=new ImapMail(w,mailCheckTicks,cfg); // cfg gets modified in constructor, so get label first
    if (label) ((Mail *)c)->setLabel(label);
  }
#endif
#ifdef USE_MIXER
  else if (strcmp(type,"mixer")==0) {
    dprintf("Found %s\n",type);
    char *mixers="vol,pcm"; // default mixers
    char *cfg=strtok(NULL,delim);
    if (!cfg) cfg=mixers;
    Mixer::create(w,1,cfg);
  }
#endif
  else if (strcmp(type,"temp")==0) {
    dprintf("Found %s\n",type);
    char *range1, *range2, *label, *filename;

    if ((range1=strtok(NULL,delim)) && (range2=strtok(NULL,delim)) &&
      (label=strtok(NULL,delim)) && (filename=strtok(NULL,delim))) {
      int min=atoi(range1);
      int max=atoi(range2);
      c=new Temperature(w,1,filename,min,max,label);
    }
  }
  return c;
}

void readConfig(Manager *w) {
  char *home=getenv("HOME");
  dprintf("HOME=%s\n",home);
  char buf[1024];
  
  strcpy(buf,home);
  strcat(buf,"/.xstatuspanelrc");
  dprintf("config=%s\n",buf);

  struct stat check;
  if (stat(buf,&check)==0) {
#ifdef USE_MAIL
      // Minimal security check to see that the .xstatuspanelrc file
      // isn't readable by anyone other than the owner.  Not great
      // security for the mail passwords, but better than nothing.  If
      // high security is desired, mail check shouldn't be used.
    if ((check.st_mode&(S_IRWXG|S_IRWXO))>0) {
      fprintf(stderr,"%s: ~/.xstatuspanelrc must be readable only by user.\n",progname);
      exit(1);
    }
#endif

    ifstream in(buf);
    if (in.is_open()) {
      while (in.getline(buf,sizeof(buf))) {
        dprintf("-->%s\n",buf);
        int i=0;
        while (isspace(buf[i])) i++;
        process(w,&buf[i]);
      }
    }
  }
  else {
    fprintf(stderr,"%s: no ~/.xstatuspanelrc file found; using default configuration.\n",progname);
    char **line=defaults;
    while (*line) {
      strcpy(buf,*line); // strtok will fail on static data
      process(w,buf);
      line++;
    }
  }
}


int main(int argc, char **argv)
{
  argProcess(argc,argv);

  RowManager w(geometry,argc,argv,appName,appClass);

  readConfig(&w);

  w.realize();
  w.eventLoop(secondsPerClockTick);
}
